% input signal
fs = 44100;
n = 0:48;
x = sin( n*pi / 6 );

% upsampling by 4
v1 = zeros( 1, 4*length(x) );		% insert 3 zeros
v1(1:4:end) = x;

% linear interpolation between 5 samples (triangular)
hi = [ 0.25, 0.5, 0.75, 1.0, 0.75, 0.5, 0.25 ];
[Hi,Fi] = freqz( hi, 1, 8192 );

% linear interpolation
v2 = conv( hi, v1 );				% filter

% linear decimation filter (mean value of 3 samples)
hd = [ 1 1 1 ] / 3;
[Hd,Fd] = freqz( hd, 1, 8192 );

% simple downsampling
w1 = v2(1:3:end);					% every 3rd value

% downsampling with linear decimation (mean value)
w2 = conv( hd, v2 );				% filter
w2 = w2(1:3:end);					% every 3rd value

% plot
figure(1);

subplot( 221 );
stem( 0:length(hi)-1, hi );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_i(n) \rightarrow' );

subplot( 222 );
plot( Fi, 20*log10(abs( Hi )) );
axis([ 0 pi -60 20 ]);
xlabel( '\Omega \rightarrow' );
ylabel( '|H_i(e^{j\Omega})| \rightarrow' );

subplot( 223 );
stem( 0:length(hd)-1, hd );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_d(n) \rightarrow' );

subplot( 224 );
plot( Fd, 20*log10(abs( Hd )) );
axis([ 0 pi -60 20 ]);
xlabel( '\Omega \rightarrow' );
ylabel( '|H_d(e^{j\Omega})| \rightarrow' );
