% input signal
fs = 44100;
n = 0:48;
x = sin( n*pi / 6 );

% upsampling by 4
v1 = zeros( 1, 4*length(x) );		% insert 3 zeros
v1(1:4:end) = x;

% linear interpolation between 5 samples (triangular)
hi = [ 0.25, 0.5, 0.75, 1.0, 0.75, 0.5, 0.25 ];
[Hi,Fi] = freqz( hi, 1, 8192 );

% linear interpolation
v2 = conv( hi, v1 );				% filter

% linear decimation filter (mean value of 3 samples)
hd = [ 1 1 1 ] / 3;
[Hd,Fd] = freqz( hd, 1, 8192 );

% simple downsampling
w1 = v2(1:3:end);					% every 3rd value

% downsampling with linear decimation (mean value)
w2 = conv( hd, v2 );				% filter
w2 = w2(1:3:end);					% every 3rd value


% plot
figure(2);

subplot( 411 );
stem( n, x );
axis([ 0 50 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'x(n) \rightarrow' );

subplot( 412 );
stem( 0:length(v1)-1, v1 );
axis([ 0 200 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v_1(n) \rightarrow' );

subplot( 413 );
stem( 0:length(v2)-1, v2 );
axis([ 0 200 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v_2(n) \rightarrow' );

subplot( 414 );
stem( 0:length(w1)-1, w1 );
hold on;
stem( 0:length(w2)-1, w2, 'g' );
hold off;
axis([ 0 66 -1.1 1.1 ]);
legend( 'w_1 without anti-aliasing', 'w_2 with anti-aliasing' );
xlabel( 'n \rightarrow' );
ylabel( 'w_i(n) \rightarrow' );
