% input signal
fs = 44100;
n = 0:48;
x = sin( n*pi / 6 );

% upsampling by 4
v1 = zeros( 1, 4*length(x) );		% insert 3 zeros
v1(1:4:end) = x;

% linear interpolation between 5 samples (triangular)
hi = [ 0.25, 0.5, 0.75, 1.0, 0.75, 0.5, 0.25 ];
[Hi,Fi] = freqz( hi, 1, 8192 );

% linear interpolation
v2 = conv( hi, v1 );				% filter

% linear decimation filter (mean value of 3 samples)
hd = [ 1 1 1 ] / 3;
[Hd,Fd] = freqz( hd, 1, 8192 );

% simple downsampling
w1 = v2(1:3:end);					% every 3rd value

% downsampling with linear decimation (mean value)
w2 = conv( hd, v2 );				% filter
w2 = w2(1:3:end);					% every 3rd value

% polyphases

% upsampling
hi1 = hi(1:4:end);		% polyphase impulse responses carry only
hi2 = hi(2:4:end);		% every n-th sample
hi3 = hi(3:4:end);
hi4 = hi(4:4:end);
hi4(2) = 0;				% all hi have the same length (=2)

disp( 'interpolation filter:' );
fprintf( '   h_i1:   %f   %f \n', hi1  );
fprintf( '   h_i2:   %f   %f \n', hi2  );
fprintf( '   h_i3:   %f   %f \n', hi3  );
fprintf( '   h_i4:   %f   %f \n', hi4  );

% downsampling
hd1 = hd(1:3:end);
hd2 = hd(2:3:end);
hd3 = hd(3:3:end);

disp( 'decimation filter:' );
fprintf( '   h_d1:   %f   %f \n', hd1 );
fprintf( '   h_d2:   %f   %f \n', hd2 );
fprintf( '   h_d3:   %f   %f \n', hd3 );


v(1,:) = conv( x, hi1 );				% independent filtering
v(2,:) = conv( x, hi2 );
v(3,:) = conv( x, hi3 );
v(4,:) = conv( x, hi4 );

v0 = reshape(v,prod(size(v)),1);		% re-read samples line-by-line
v0 = [ v0; 0 ];							% add one zero sample

w(1,:) = conv( v0(1:3:end), hd1 );		% filter each 3rd sample
w(2,:) = conv( v0(2:3:end), hd2 );
w(3,:) = conv( v0(3:3:end), hd3 );

w0 = sum(w,1);							% sum up all rows

% plot
figure(4);

subplot( 341 );
stem( 0:length(hi1)-1, hi1 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{i,1}(n) \rightarrow' );

subplot( 342 );
stem( 0:length(hi2)-1, hi2 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{i,2}(n) \rightarrow' );

subplot( 343 );
stem( 0:length(hi3)-1, hi3 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{i,3}(n) \rightarrow' );

subplot( 344 );
stem( 0:length(hi4)-1, hi4 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{i,4}(n) \rightarrow' );

subplot( 345 );
stem( 0:length(v(1,:))-1, v(1,:) );
axis([ 0 12 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v_1(n) \rightarrow' );

subplot( 346 );
stem( 0:length(v(2,:))-1, v(2,:) );
axis([ 0 12 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v_2(n) \rightarrow' );

subplot( 347 );
stem( 0:length(v(3,:))-1, v(3,:) );
axis([ 0 12 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v_3(n) \rightarrow' );

subplot( 348 );
stem( 0:length(v(4,:))-1, v(4,:) );
axis([ 0 12 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v_4(n) \rightarrow' );

subplot( 313 );
stem( 0:length(v0)-1, v0 );
axis([ 0 200 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'v(n) \rightarrow' );

% plot
figure(5);

subplot( 331 );
stem( 0:length(hd1)-1, hd1 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{d,1}(n) \rightarrow' );

subplot( 332 );
stem( 0:length(hd2)-1, hd2 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{d,2}(n) \rightarrow' );

subplot( 333 );
stem( 0:length(hd3)-1, hd3 );
axis([ -1 7 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'h_{d,3}(n) \rightarrow' );

subplot( 334 );
stem( 0:length(w(1,:))-1, w(1,:) );
axis([ 0 16 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'w_1(n) \rightarrow' );

subplot( 335 );
stem( 0:length(w(2,:))-1, w(2,:) );
axis([ 0 16 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'w_2(n) \rightarrow' );

subplot( 336 );
stem( 0:length(w(3,:))-1, w(3,:) );
axis([ 0 16 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'w_3(n) \rightarrow' );

subplot( 313 );
stem( 0:length(w0)-1, w0 );
axis([ 0 67 -1.1 1.1 ]);
xlabel( 'n \rightarrow' );
ylabel( 'w(n) \rightarrow' );
